@@ -1,6 +1,6 @@
 package Bio::DB::USeq;
 
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 
 =head1 NAME
 
@@ -142,20 +142,16 @@ cache for future queries.
 
 No support for genomic sequence is included. Users who need access to 
 genomic sequence should seek an alternative C<BioPerl> adaptor, such as 
-C<Bio::DB::Fasta>.
+L<Bio::DB::Fasta>.
 
 Useq files do not have a native concept of type, primary_tag, or source 
 attributes, as expected with GFF-based database adaptors. The features 
 method does support special types (see below).
 
-Currently, C<Bio::DB::USeq> can only parse files that do not contain 
-text strings. This is a current technical limitation that should 
-be overcome.
-
 =head2 Requirements
 
 The adaptor is a Perl-only implementation. It only requires the 
-C<Archive::Zip> module for opening and reading the useq file archive. 
+L<Archive::Zip> module for opening and reading the useq file archive. 
 C<BioPerl> is required for working with SeqFeatures objects generated 
 from useq file observations.
 
@@ -197,7 +193,7 @@ collecting data.
 
 =item zip
 
-Return the C<Archive::Zip> object representing the useq file archive.
+Return the L<Archive::Zip> object representing the useq file archive.
 
 =back
 
@@ -279,7 +275,7 @@ more keyed values.
     );
 
 If the -iterator argument is supplied with a true value, then an 
-iterator object is returned. See L<get_seq_stream> for details.
+iterator object is returned. See get_seq_stream() for details.
 
 C<Bio::DB::USeq> supports four different feature types. Feature 
 types may be specified using the -type argument.
@@ -318,9 +314,9 @@ returned representing each strand.
 Each object contains an array representing scores across 
 the requested coordinates. This object is designed to be 
 backwards compatible with coverage features from the 
-C<Bio::DB::Sam> adaptor for use with C<Bio::Graphics> and GBrowse. 
+L<Bio::DB::Sam> adaptor for use with L<Bio::Graphics> and GBrowse. 
 Note that only scores are returned, not a true depth coverage 
-in the sense of the C<Bio::DB::Sam> coverage types. 
+in the sense of the L<Bio::DB::Sam> coverage types. 
 
 By default, the wiggle or coverage array is provided at 1 
 bp resolution. To improve efficiency with large regions, 
@@ -376,8 +372,8 @@ calculated.
   }
 
 Statistical summaries are equivalent to those generated by the 
-C<Bio::DB::BigWig> adaptor and may be used interchangeably. They 
-are compatible with the C<Bio::Graphics> modules.
+L<Bio::DB::BigWig> adaptor and may be used interchangeably. They 
+are compatible with the L<Bio::Graphics> modules.
 
 Summary objects may also be obtained with a get_seq_stream() 
 iterator object.
@@ -554,7 +550,7 @@ information.
 =head2 Bio::DB::USeq::Wiggle objects
 
 These are SeqFeatureI compliant objects for backwards compatibility with 
-C<Bio::Graphics> and GBrowse. They support the wiggle() and coverage() 
+L<Bio::Graphics> and GBrowse. They support the wiggle() and coverage() 
 methods, which returns an array of scores over the interrogated region. By 
 default, the array is equal to the length of the region (1 bp resolution), 
 or may be limited to a specified number of bins for efficiency. See the 
@@ -587,8 +583,8 @@ for details.
 =head2 Bio::DB::USeq::Summary objects
 
 These are SeqFeatureI compliant Summary objects, similar to those 
-generated by the C<Bio::DB::BigWig> database adaptor. As such, they are 
-compatible with C<Bio::Graphics> and GBrowse. 
+generated by the L<Bio::DB::BigWig> database adaptor. As such, they are 
+compatible with L<Bio::Graphics> and GBrowse. 
 
 Summary objects can generate statistical summaries over a specified 
 number of bins (default is 1 bin, or the entire requested region). 
@@ -637,7 +633,7 @@ for details.
 Three subroutine functions are available for export to assist 
 in calculating the mean, variance, and standard deviation from 
 statistical summaries. These functions are identical to those 
-from the C<Bio::DB::BigWig> adaptor and may be used interchangeably. 
+from the L<Bio::DB::BigWig> adaptor and may be used interchangeably. 
 
 They are not exported by default; they must explicitly listed.
     
@@ -719,7 +715,7 @@ Return the number of observations recorded in the slice interval.
 
 =head1 GBROWSE COMPATIBILITY
 
-The USeq adaptor has support for C<Bio::Graphics> and GBrowse. 
+The USeq adaptor has support for L<Bio::Graphics> and GBrowse. 
 It will work with the segments glyph for intervals, 
 the wiggle_xyplot glyph for displaying mean scores, and the 
 wiggle_whiskers glyph for displaying detailed statistics.
@@ -764,10 +760,9 @@ than simple bins of mean values as with the wiggle feature type.
 Because the C<Bio::DB::USeq> is implemented as a Perl-only module, 
 performance is subject to the limitations of Perl execution itself and 
 the size of the data that needs to be parsed. In general, using the 
-wiggle feature type is faster than the summary feature type. Very 
-large USeq files (half a gigabyte or larger) may see better 
-performance when converted to a BigWig file, especially when viewing 
-large intervals.
+wiggle feature type is slightly faster than the summary feature type. 
+Very large USeq files may see better performance when converted to a 
+BigWig file, particularly when viewing large intervals.
 =cut
 
 
@@ -1259,6 +1254,7 @@ sub scores {
 	# determine which slices to retrieve
 	my @slices = $self->_translate_coordinates_to_slices(
 		$seq_id, $start, $stop, $strand);
+	return unless @slices;
 	$self->_clear_buffer(@slices);
 	
 	# collect the scores from each of the requested slices
@@ -1326,13 +1322,6 @@ sub _parse_members {
 			next;
 		}
 		
-		# Check for text strings
-		# The perl unpack function cannot handle Java binary text strings
-		if ($extension =~ /t/) {
-			carp " Cannot parse USeq archives containing text!\n";
-			return;
-		}
-		
 		# check stranded data
 		unless (defined $self->{'stranded'}) {
 			if ($strand eq '.') {
@@ -1494,19 +1483,33 @@ sub _load_slice {
 	$self->{'buffer'}{$slice} = [];
 	
 	my $type = $self->slice_type($slice);
-	if ($type eq 's')      { $self->_load_s_slice($slice) }
-	elsif ($type eq 'sf')  { $self->_load_sf_slice($slice) }
-	elsif ($type eq 'ss')  { $self->_load_ss_slice($slice) }
-	elsif ($type eq 'ssf') { $self->_load_ssf_slice($slice) }
-	elsif ($type eq 'i')   { $self->_load_i_slice($slice) }
-	elsif ($type eq 'ii')  { $self->_load_ii_slice($slice) }
-	elsif ($type eq 'if')  { $self->_load_if_slice($slice) }
-	elsif ($type eq 'iif') { $self->_load_iif_slice($slice) }
-	elsif ($type eq 'is')  { $self->_load_is_slice($slice) }
-	elsif ($type eq 'isf') { $self->_load_isf_slice($slice) }
+	if    ($type eq 's')    { $self->_load_s_slice($slice) }
+	elsif ($type eq 'sf')   { $self->_load_sf_slice($slice) }
+	elsif ($type eq 'st')   { $self->_load_st_slice($slice) }
+	elsif ($type eq 'ss')   { $self->_load_ss_slice($slice) }
+	elsif ($type eq 'ssf')  { $self->_load_ssf_slice($slice) }
+	elsif ($type eq 'sst')  { $self->_load_sst_slice($slice) }
+	elsif ($type eq 'ssft') { $self->_load_ssft_slice($slice) }
+	elsif ($type eq 'i')    { $self->_load_i_slice($slice) }
+	elsif ($type eq 'if')   { $self->_load_if_slice($slice) }
+	elsif ($type eq 'it')   { $self->_load_it_slice($slice) }
+	elsif ($type eq 'ii')   { $self->_load_ii_slice($slice) }
+	elsif ($type eq 'iif')  { $self->_load_iif_slice($slice) }
+	elsif ($type eq 'iit')  { $self->_load_iit_slice($slice) }
+	elsif ($type eq 'iift') { $self->_load_iift_slice($slice) }
+	elsif ($type eq 'is')   { $self->_load_is_slice($slice) }
+	elsif ($type eq 'isf')  { $self->_load_isf_slice($slice) }
+	elsif ($type eq 'ist')  { $self->_load_ist_slice($slice) }
+	elsif ($type eq 'isft') { $self->_load_isft_slice($slice) }
 	else {
 		croak "unable to load slice '$slice'! Unsupported slice type $type\n";
 	}
+	
+	# sanity check
+	if (scalar @{ $self->{buffer}{$slice} } != $self->slice_obs_number($slice)) {
+		croak "slice load failed! Only loaded ", scalar @{ $self->{buffer}{$slice} }, 
+			" observations when expecting ", $self->slice_obs_number($slice), "!\n";
+	}
 }
 
 
@@ -1550,6 +1553,36 @@ sub _load_sf_slice {
 	}
 }
 
+sub _load_st_slice {
+	my $self = shift;
+	my $slice = shift;
+	
+	# convert the unpacked data into start, stop, (score), text
+	
+	# load the slice contents
+	my $contents = $self->zip->contents($slice);
+	
+	# first observation
+	my $i = 8; # go ahead and set index up to first observation text
+	my (undef, $position, $t) = unpack('si>s>', substr($contents, 0, $i));
+		# initial null, position, text_length
+	$position += 1;
+	my $text = unpack("A$t", substr($contents, $i, $t));
+	push @{ $self->{buffer}{$slice} }, [$position, $position, undef, $text];
+	$i += $t;
+	
+	# remaining observations
+	my $p; # position offset
+	while ($i < CORE::length($contents)) {
+		my ($p, $t) = unpack('s>s>', substr($contents, $i, 4));
+		$i += 4;
+		$position += $p + 32768;
+		$text = unpack("A$t", substr($contents, $i, $t));
+		push @{ $self->{buffer}{$slice} }, [$position, $position, undef, $text];
+		$i += $t;
+	}
+}
+
 sub _load_ss_slice {
 	my $self = shift;
 	my $slice = shift;
@@ -1594,6 +1627,62 @@ sub _load_ssf_slice {
 	}
 }
 
+sub _load_sst_slice {
+	my $self = shift;
+	my $slice = shift;
+	my $contents = $self->zip->contents($slice);
+	
+	# first observation
+	my $i = 10; # go ahead and set index up to first observation text
+	my (undef, $position, $l, $t) = unpack('si>s>s>', substr($contents, 0, $i));
+		# initial null, position, length, text_length
+	$position += 1; # interbase to base
+	my $position2 = $position + $l + 32767; # 32768 - 1
+	my $text = unpack("A$t", substr($contents, $i, $t));
+	push @{ $self->{buffer}{$slice} }, [$position, $position2, undef, $text];
+	$i += $t;
+	
+	# remaining observations
+	my $p; # position offset
+	while ($i < CORE::length($contents)) {
+		($p, $l, $t) = unpack('s>s>s>', substr($contents, $i, 6));
+		$i += 6;
+		$position += $p + 32768;
+		$position2 = $position + $l + 32767;
+		$text = unpack("A$t", substr($contents, $i, $t));
+		push @{ $self->{buffer}{$slice} }, [$position, $position2, undef, $text];
+		$i += $t;
+	}
+}
+
+sub _load_ssft_slice {
+	my $self = shift;
+	my $slice = shift;
+	my $contents = $self->zip->contents($slice);
+	
+	# first observation
+	my $i = 14; # go ahead and set index up to first observation text
+	my (undef, $position, $l, $s, $t) = unpack('si>s>f>s>', substr($contents, 0, $i));
+		# initial null, position, length, score, text_length
+	$position += 1; # interbase to base
+	my $position2 = $position + $l + 32767; # 32768 - 1
+	my $text = unpack("A$t", substr($contents, $i, $t));
+	push @{ $self->{buffer}{$slice} }, [$position, $position2, $s, $text];
+	$i += $t;
+	
+	# remaining observations
+	my $p; # position offset
+	while ($i < CORE::length($contents)) {
+		($p, $l, $s, $t) = unpack('s>s>f>s>', substr($contents, $i, 10));
+		$i += 10;
+		$position += $p + 32768;
+		$position2 = $position + $l + 32767;
+		$text = unpack("A$t", substr($contents, $i, $t));
+		push @{ $self->{buffer}{$slice} }, [$position, $position2, $s, $text];
+		$i += $t;
+	}
+}
+
 sub _load_i_slice {
 	my $self = shift;
 	my $slice = shift;
@@ -1614,6 +1703,56 @@ sub _load_i_slice {
 	}
 }
 
+sub _load_if_slice {
+	my $self = shift;
+	my $slice = shift;
+	
+	# unpack the data slice zip member
+	my $number = $self->slice_obs_number($slice);
+	my ($null, @data) = unpack("si>f>(i>f>)$number", $self->zip->contents($slice) );
+	
+	# convert the unpacked data into start, stop, score
+	# first observation
+	my $position = (shift @data) + 1;
+	push @{ $self->{buffer}{$slice} }, [$position, $position, shift(@data)];
+	
+	# remaining observations
+	while (@data) {
+		$position += (shift @data);
+		push @{ $self->{buffer}{$slice} }, [$position, $position, shift(@data)];
+	}
+}
+
+sub _load_it_slice {
+	my $self = shift;
+	my $slice = shift;
+	
+	# convert the unpacked data into start, stop, (score), text
+	
+	# load the slice contents
+	my $contents = $self->zip->contents($slice);
+	
+	# first observation
+	my $i = 10; # go ahead and set index up to first observation text
+	my (undef, $position, $t) = unpack('si>i>', substr($contents, 0, $i));
+		# initial null, position, text_length
+	$position += 1;
+	my $text = unpack("A$t", substr($contents, $i, $t));
+	push @{ $self->{buffer}{$slice} }, [$position, $position, undef, $text];
+	$i += $t;
+	
+	# remaining observations
+	my $p; # position offset
+	while ($i < CORE::length($contents)) {
+		($p, $t) = unpack('i>s>', substr($contents, $i, 6));
+		$i += 6; 
+		$position += $p + 32768;
+		$text = unpack("A$t", substr($contents, $i, $t));
+		push @{ $self->{buffer}{$slice} }, [$position, $position, undef, $text];
+		$i += $t;
+	}
+}
+
 sub _load_ii_slice {
 	my $self = shift;
 	my $slice = shift;
@@ -1636,26 +1775,6 @@ sub _load_ii_slice {
 	}
 }
 
-sub _load_if_slice {
-	my $self = shift;
-	my $slice = shift;
-	
-	# unpack the data slice zip member
-	my $number = $self->slice_obs_number($slice);
-	my ($null, @data) = unpack("si>f>(i>f>)$number", $self->zip->contents($slice) );
-	
-	# convert the unpacked data into start, stop, score
-	# first observation
-	my $position = (shift @data) + 1;
-	push @{ $self->{buffer}{$slice} }, [$position, $position, shift(@data)];
-	
-	# remaining observations
-	while (@data) {
-		$position += (shift @data);
-		push @{ $self->{buffer}{$slice} }, [$position, $position, shift(@data)];
-	}
-}
-
 sub _load_iif_slice {
 	my $self = shift;
 	my $slice = shift;
@@ -1678,6 +1797,62 @@ sub _load_iif_slice {
 	}
 }
 
+sub _load_iit_slice {
+	my $self = shift;
+	my $slice = shift;
+	my $contents = $self->zip->contents($slice);
+	
+	# first observation
+	my $i = 12; # go ahead and set index up to first observation text
+	my (undef, $position, $l, $t) = unpack('si>i>s>', substr($contents, 0, $i));
+		# initial null, position, length, text_length
+	$position += 1; # interbase to base
+	my $position2 = $position + $l - 1;
+	my $text = unpack("A$t", substr($contents, $i, $t));
+	push @{ $self->{buffer}{$slice} }, [$position, $position2, undef, $text];
+	$i += $t;
+	
+	# remaining observations
+	my $p; # position offset
+	while ($i < CORE::length($contents)) {
+		($p, $l, $t) = unpack('i>i>s>', substr($contents, $i, 10));
+		$i += 10;
+		$position += $p;
+		$position2 = $position + $l - 1;
+		$text = unpack("A$t", substr($contents, $i, $t));
+		push @{ $self->{buffer}{$slice} }, [$position, $position2, undef, $text];
+		$i += $t;
+	}
+}
+
+sub _load_iift_slice {
+	my $self = shift;
+	my $slice = shift;
+	my $contents = $self->zip->contents($slice);
+	
+	# first observation
+	my $i = 16; # go ahead and set index up to first observation text
+	my (undef, $position, $l, $s, $t) = unpack('si>i>f>s>', substr($contents, 0, $i));
+		# initial null, position, length, score, text_length
+	$position += 1; # interbase to base
+	my $position2 = $position + $l - 1;
+	my $text = unpack("A$t", substr($contents, $i, $t));
+	push @{ $self->{buffer}{$slice} }, [$position, $position2, $s, $text];
+	$i += $t;
+	
+	# remaining observations
+	my $p; # position offset
+	while ($i < CORE::length($contents)) {
+		($p, $l, $s, $t) = unpack('i>i>f>s>', substr($contents, $i, 14));
+		$i += 14;
+		$position += $p;
+		$position2 = $position + $l - 1;
+		$text = unpack("A$t", substr($contents, $i, $t));
+		push @{ $self->{buffer}{$slice} }, [$position, $position2, $s, $text];
+		$i += $t;
+	}
+}
+
 sub _load_is_slice {
 	my $self = shift;
 	my $slice = shift;
@@ -1722,6 +1897,62 @@ sub _load_isf_slice {
 	}
 }
 
+sub _load_ist_slice {
+	my $self = shift;
+	my $slice = shift;
+	my $contents = $self->zip->contents($slice);
+	
+	# first observation
+	my $i = 10; # go ahead and set index up to first observation text
+	my (undef, $position, $l, $t) = unpack('si>s>s>', substr($contents, 0, $i));
+		# initial null, position, length, text_length
+	$position += 1; # interbase to base
+	my $position2 = $position + $l + 32767; # 32768 - 1
+	my $text = unpack("A$t", substr($contents, $i, $t));
+	push @{ $self->{buffer}{$slice} }, [$position, $position2, undef, $text];
+	$i += $t;
+	
+	# remaining observations
+	my $p; # position offset
+	while ($i < CORE::length($contents)) {
+		($p, $l, $t) = unpack('i>s>s>', substr($contents, $i, 8));
+		$i += 8;
+		$position += $p;
+		$position2 = $position + $l + 32767;
+		$text = unpack("A$t", substr($contents, $i, $t));
+		push @{ $self->{buffer}{$slice} }, [$position, $position2, undef, $text];
+		$i += $t;
+	}
+}
+
+sub _load_isft_slice {
+	my $self = shift;
+	my $slice = shift;
+	my $contents = $self->zip->contents($slice);
+	
+	# first observation
+	my $i = 14; # go ahead and set index up to first observation text
+	my (undef, $position, $l, $s, $t) = unpack('si>s>f>s>', substr($contents, 0, $i));
+		# initial null, position, length, score, text_length
+	$position += 1; # interbase to base
+	my $position2 = $position + $l + 32767; # 32768 - 1
+	my $text = unpack("A$t", substr($contents, $i, $t));
+	push @{ $self->{buffer}{$slice} }, [$position, $position2, $s, $text];
+	$i += $t;
+	
+	# remaining observations
+	my $p; # position offset
+	while ($i < CORE::length($contents)) {
+		($p, $l, $s, $t) = unpack('i>s>f>s>', substr($contents, $i, 12));
+		$i += 12;
+		$position += $p;
+		$position2 = $position + $l + 32767;
+		$text = unpack("A$t", substr($contents, $i, $t));
+		push @{ $self->{buffer}{$slice} }, [$position, $position2, $s, $text];
+		$i += $t;
+	}
+}
+
 sub _scores {
 	my $self = shift;
 	my ($start, $stop, $slices) = @_;
@@ -2192,7 +2423,6 @@ sub _next_region {
 		}
 		
 		# no luck, prepare for next observation
-		$index++;
 		if ($index == $last_index) {
 			# reset the slice, although unlikely there are anymore
 			$slice = shift @{ $self->{'slices'} } || undef;
@@ -2204,6 +2434,9 @@ sub _next_region {
 			$index = 0;
 			$self->{current_index} = $index;
 		}
+		else {
+			$index++;
+		}
 	}
 	return;
 }
@@ -2358,7 +2591,7 @@ sub wiggle {
 # Borrowed from Bio::SeqFeature::Coverage from Bio::DB::Sam
 sub gff3_string {
     my $self = shift;
-    my $gff3 = $self->SUPER::gff3_string;
+    my $gff3 = $self->SUPER::gff3_string(@_);
     my $coverage = $self->escape(join(',',$self->coverage));
     $gff3 =~ s/coverage=[^;]+/coverage=$coverage/g;
     return $gff3;
@@ -2523,14 +2756,6 @@ sub get_seq_stream {
 	);
 }
 
-=head1 TODO
-
-Interpret text fields from slices containing text. The text fields are 
-written using Java's WriteUTF function, which is a modified UTF-8 format, 
-wherein a short is first written to record the length of the subsequent 
-UTF-8 string. This is going to require a complicated unpack that loops 
-through the binary string one observation at a time. Ugh.
-
 =head1 AUTHOR
 
  Timothy J. Parnell, PhD
